<?php

namespace Drupal\mercury_layouts\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\style_options\Plugin\Layout\StyleOptionLayoutPlugin;

/**
 * Defines an Accordion / Tabs base layout plugin.
 */
class DynamicRegionsLayoutBase extends StyleOptionLayoutPlugin {

  /**
   * The item label (i.e. Tab or Accordion Item).
   *
   * @var string
   */
  protected $regionLabel = 'Group';

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->pluginDefinition->setRegions($this->getConfiguration()['layout_regions']);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    $form['layout_regions'] = $this->getConfiguration()['layout_regions'];
    foreach ($this->getConfiguration()['layout_regions'] as $region_name => $region_data) {
      $form['layout_regions'][$region_name]['label'] = [
        '#type' => 'hidden',
        '#default_value' => $region_data['label'],
      ];
    }
    return parent::buildConfigurationForm($form, $form_state);

  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $parent = parent::defaultConfiguration();
    if (empty($this->configuration['layout_regions'])) {
      return $parent + [
        'layout_regions' => [
          'region_1' => ['label' => $this->regionLabel . ' 1'],
          'region_2' => ['label' => $this->regionLabel . ' 2'],
        ],
      ];
    }
    return $parent;
  }

  /**
   * {@inheritDoc}
   */
  public function build(array $regions) {
    $this->pluginDefinition->setRegions($this->getConfiguration()['layout_regions']);
    $build = parent::build($regions);
    $build['#region_label'] = $this->regionLabel;
    return $build;
  }

  /**
   * Adds edit / delete buttons for regions.
   *
   * @param array $build
   *   The build array.
   * @param array $regions
   *   The regions.
   */
  protected function buildRegionControls(array &$build, array $regions) {
    foreach (array_keys($regions) as $region) {
      $build[$region]['#controls'] = [
        '#theme' => 'mecury_layouts_region_controls',
        '#region' => $this->layoutId() . '--' . $region,
      ];
    }
  }

  /**
   * Returns the layout paragraphs layout instance ID.
   *
   * @return string
   *   The layout paragraphs layout instance ID.
   */
  protected function layoutId() {
    $config = $this->getConfiguration();
    if (!empty($config['layout_paragraphs_section'])) {
      $entity = $config['layout_paragraphs_section']->getEntity();
      return !empty($entity->_layoutParagraphsLayout) ?
        $entity->_layoutParagraphsLayout->id() :
        NULL;
    }
  }

  /**
   * {@inheritDoc}
   */
  public function processForm(array $element, FormStateInterface $form_state) {
    $this->pluginDefinition->setRegions($this->getConfiguration()['layout_regions']);
    return parent::processForm($element, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->pluginDefinition->setRegions($this->getConfiguration()['layout_regions']);
    return parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  protected function getRegionContextDefinitions() {
    $this->pluginDefinition->setRegions($this->getConfiguration()['layout_regions']);
    return parent::getRegionContextDefinitions();
  }

}
